##############################################################################
#
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
#
# Copyright (c) Digital Creations.  All rights reserved.
#
# This license has been certified as Open Source(tm).
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
#
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
#
##############################################################################
from OFS.Folder import Folder

from Products.CMFCore.MemberDataTool import MemberData as CMFMemberData

from Products.CMFPropertyCore.LinkPropertyManager import LinkPropertyManager
from Products.CMFPropertyCore.PropertyFolder import PropertyFolder, beforeCreate, afterCreate, InitializeClassPropertyFolder

import UsrManOrg

# Import permission names
import MemberDataPermissions

# Factory type information -- makes MemberDatas objects play nicely
# with the Types  (portal_types)
# template
factory_type_information = (
    { 'id'                : 'MemberData'
      , 'meta_type'       : 'CMF UsrManOrg MemberData'
      , 'description'     : """\
UsrManOrg MemberData object."""
      , 'icon'            : 'member_data_icon.gif'
      , 'product'         : 'CMFUsrManOrg'
      , 'factory'         : 'manage_addMemberData'
      , 'immediate_view'  : 'member_data_edit_form'
      , 'actions'         :
      ( { 'name'          : 'View'
          , 'action'      : 'member_data_view'
          , 'permissions' : (MemberDataPermissions.View,)
          }
      , { 'name'          : 'Edit'
          , 'action'      : 'member_data_edit_form'
          , 'permissions' : (MemberDataPermissions.Modify,)
          }
      , { 'name'          : 'Metadata'
          , 'action'      : 'metadata_edit_form'
          , 'permissions' : (MemberDataPermissions.Modify,)
          }
        )
      , 'allowed_content_types' : ('Man','Name','Tel','Loc','Uri',)
      }
    ,)

def manage_addMemberData(self, id=None, title=None, parent=None, REQUEST=None, tool=None):
    """
    Create an empty Member Data.
    """
    id = beforeCreate(id, 'mdata', REQUEST)
    obj = MemberData(tool,id,title,parent)
    afterCreate(self, id, obj, REQUEST)
    return id

class MemberData(PropertyFolder, CMFMemberData):
    """
    MemberData contains a single CMF member's data.
    """
    meta_type='CMF UsrManOrg MemberData'
    
    manage_options=Folder.manage_options
    
    def __init__(self, tool, id, title=None, parent=None):
        PropertyFolder.__init__(self, id, title, parent)
        UsrManOrg.extend_propertiesUsr(self)
        self._tool = tool

CMFMemberData.type_converters = LinkPropertyManager.type_converters

# Initialize the MemberData class, setting up security.
InitializeClassPropertyFolder(MemberData
                              , MemberDataPermissions.View
                              , MemberDataPermissions.Modify
                              , MemberDataPermissions.ListContents
                              , 1
                              , 1
                              )
