##############################################################################
#
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
#
# Copyright (c) Digital Creations.  All rights reserved.
#
# This license has been certified as Open Source(tm).
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
#
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
#
##############################################################################
from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner, aq_parent, aq_base
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.MemberDataTool import MemberDataTool as CMFMemberDataTool
from Products.CMFCore.MemberDataTool import MemberData as CMFMemberData

from Products.CMFPropertyCore.LinkPropertyManager import LinkPropertyManager
from Products.CMFPropertyCore.PropertyBTreeFolder import _copyObj, PropertyBTreeFolder, InitializeClassPropertyBTreeFolder

import UsrManOrg
from MemberData import manage_addMemberData, MemberData

# import permissions
import MemberDataToolPermissions
import MemberDataPermissions

class MemberDataTool(PropertyBTreeFolder, CMFMemberDataTool):
    """
    MemberDataTools contain CMF member data.
    """
    meta_type='CMF MemberDataTool'

    security = ClassSecurityInfo()
    
    manage_options=( ( { 'label' : 'Overview'
                         , 'action' : 'manage_overview'
                         }
                       , { 'label' : 'Member Contents'
                           , 'action' : 'manage_showContents'
                           }
                       )
                     + PropertyBTreeFolder.manage_options
                     )

    # Simply a hack to hardwire index_html
    index_html = CMFMemberDataTool.manage_overview
    CMFMemberDataTool.manage_overview._setName("manage_overview")

    #
    #   ZMI methods
    #
    security.declareProtected( MemberDataToolPermissions.Manage
                               , 'manage_overview' )
    security.declareProtected( MemberDataToolPermissions.ViewManagementScreens
                               , 'manage_showContents')
    security.declareProtected( MemberDataToolPermissions.ViewManagementScreens
                               , 'getContentsInformation')

    def __init__(self, id=CMFMemberDataTool.id, title=None, parent=None):
        PropertyBTreeFolder.__init__(self, id, title, parent)
        UsrManOrg.extend_propertiesUsr(self)
        # required for CMFMemberDataTool
        self._members = self._tree

    def manage_afterAdd_DISABLE_(self, item, container):
        '''
        Copy old memberdata objects and setup registerMemberData hook.
        However, do not copy any of the properties of the old memberdata object.
        '''
        id = CMFMemberDataTool.id
        # safety belt for copy and rename operations
        if item.id != id:
            memberdata = container._getOb(id, None)
            if memberdata != None and memberdata != item:
                # copy old memberdata objects
                members = memberdata._members
                for key,obj in members.items():
                    # setup registerMemberData hook
                    obj._tool = item
                    item._setObject(key, obj)
                # delete object
                container.manage_delObjects([id])
            # rename to memberdata object
            container.manage_renameObject(item.id,id)
            memberdata = container._getOb(id)

            # set Portal Type
            if hasattr(memberdata, '_setPortalTypeName'):
                memberdata._setPortalTypeName('Folder')
            # setup permissions
            memberdata.manage_permission(MemberDataPermissions.Add,['Manager',],1)

    def pruneMemberDataContents(self):
        '''
        Compare the user IDs stored in the member data
        tool with the list in the actual underlying acl_users
        and delete anything not in acl_users
        '''
        membertool= getToolByName(self, 'portal_membership')
        members   = self._members
        user_list = membertool.listMemberIds()

        for tuple in members.items():
            member_name = tuple[0]
            member_obj  = tuple[1]
            if member_name not in user_list:
                self.manage_delObjects([member_name])

    security.declarePrivate('registerMemberData')
    def registerMemberData(self, m, id):
        '''
        Adds the given member data to the _members dict.
        This is done as late as possible to avoid side effect
        transactions and to reduce the necessary number of
        entries.
        '''
        # if necessary delete old object
        if self._getOb(id, None) is not None:
            self._delObject(id)
        # create new object
        apply(manage_addMemberData, (self,id,id), {'tool':self})
        obj = self._getOb(id)
        if hasattr(obj, '_setPortalTypeName'):
            obj._setPortalTypeName('MemberData')
        # delete notify hook
        if hasattr(obj,'_tool'):
            delattr(obj,'_tool')
        # transfer properties
        setattr(m,'title',id)
        apply(LinkPropertyManager.manage_changeProperties, (obj,), m.__dict__)
        # set up ownership
        membertool= getToolByName(self, 'portal_membership')
        member = membertool.getMemberById(id)
        if hasattr(getattr(member, 'aq_base', member), 'getUser'):
            muser = member.getUser()
        else:
            muser = member
        if muser is not None:
            obj.changeOwnership(muser)
        obj.manage_setLocalRoles(id, ['Owner'])

CMFMemberDataTool.type_converters = LinkPropertyManager.type_converters

# Initialize the MemberDataTool class, setting up security.
InitializeClassPropertyBTreeFolder(MemberDataTool
                                   , MemberDataToolPermissions.View
                                   , MemberDataToolPermissions.Modify
                                   , MemberDataToolPermissions.ListContents
                                   , 1
                                   , 1
                                   )
